@@ -4,13 +4,16 @@
 """Evohome RF - a RAMSES-II protocol decoder & analyser."""
 
 from abc import ABCMeta, abstractmethod
-import asyncio
 import logging
 from typing import Any, Dict, Optional
 
 from .command import Command, Priority
 from .const import (
     _dev_mode_,
+    ATTR_HEAT_DEMAND,
+    ATTR_SETPOINT,
+    ATTR_TEMP,
+    ATTR_WINDOW_OPEN,
     DEVICE_HAS_BATTERY,
     DEVICE_TABLE,
     DEVICE_TYPES,
@@ -20,9 +23,9 @@ from .const import (
     DISCOVER_ALL,
     DOMAIN_TYPE_MAP,
 )
+
 from .exceptions import CorruptStateError
-from .helpers import slugify_string as slugify, dev_id_to_hex
-from .ramses import HINTS_DEVICE_TYPES as HINTS_DEVICES
+from .helpers import dev_id_to_hex
 
 DEV_MODE = _dev_mode_
 
@@ -31,19 +34,13 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-def _payload(msg, key=None) -> Optional[Any]:
-    if msg and not msg.is_expired:
-        if key:
-            return msg.payload.get(key)
-        return {k: v for k, v in msg.payload.items() if k[:1] != "_"}
-
-
 class Entity:
     """The Device/Zone base class."""
 
     def __init__(self, gwy) -> None:
-        self._gwy = gwy
+        self._loop = gwy._loop
 
+        self._gwy = gwy
         self.id = None
 
         self._msgs = {}
@@ -67,9 +64,9 @@ class Entity:
             }
 
     def _handle_msg(self, msg) -> None:  # TODO: beware, this is a mess
-        if "domain_id" in msg.payload:  # isinstance(msg.payload, dict) and
-            self._domain[msg.payload["domain_id"]] = {msg.code: msg}  # 01/02/23
-            return
+        # if "domain_id" in msg.payload:  # isinstance(msg.payload, dict) and
+        #     self._domain[msg.payload["domain_id"]] = {msg.code: msg}  # 01/02/23
+        #     return
 
         if msg.verb == " W":
             if msg.code in self._msgs and self._msgs[msg.code].verb != msg.verb:
@@ -86,15 +83,39 @@ class Entity:
     def _send_cmd(self, code, dest, payload, **kwargs) -> None:
         self._msgs.pop(code, None)  # remove the old one, so we can tell if RP'd rcvd
 
-        cmd = Command(kwargs.pop("verb", "RQ"), dest, code, payload, **kwargs)
-        try:  # TODO: convert create_task to run_coroutine_threadsafe
-            asyncio.create_task(
-                self._gwy.msg_protocol.send_data(cmd),
-            )
-        except RuntimeError:
-            asyncio.run_coroutine_threadsafe(
-                self._gwy.msg_protocol.send_data(cmd), self._gwy._loop
+        self._gwy.send_cmd(
+            Command(kwargs.pop("verb", "RQ"), dest, code, payload, **kwargs)
+        )
+
+    def _msg_payload(self, msg, key=None) -> Optional[Any]:
+        if msg:
+            if key:
+                return msg.payload.get(key)
+            return {k: v for k, v in msg.payload.items() if k[:1] != "_"}
+
+    def _msg_expired(self, msg_name: str) -> Optional[bool]:
+        attr = f"_{msg_name}"
+        if not hasattr(self, attr):
+            _LOGGER.error("%s: is not tracking %s msgs", self, msg_name)
+            return
+
+        msg = getattr(self, f"_{msg_name}")
+        if not msg:
+            _LOGGER.warning("%s: has no valid %s msg", self, msg_name)
+        # elif msg_name != RAMSES_CODES[msg.code][NAME]:
+        #     _LOGGER.warning(
+        #         "%s: Message(%s) doesn't match name: %s",
+        #         self,
+        #         msg._pkt._header,
+        #         msg_name,
+        #     )
+        #     assert False, msg.code
+        elif msg.is_expired:
+            _LOGGER.warning(
+                "%s: Message(%s) has expired (%s)", self, msg._pkt._header, attr
             )
+        else:
+            return True
 
     @property
     def _pkt_codes(self) -> list:
@@ -137,7 +158,7 @@ class DeviceBase(Entity, metaclass=ABCMeta):
             self._is_sensor = None
 
         self._zone = None
-        self._domain = {}
+        # self._domain = {}  # TODO
 
     def __repr__(self) -> str:
         return f"{self.id} ({DEVICE_TYPES.get(self.type)})"
@@ -145,21 +166,25 @@ class DeviceBase(Entity, metaclass=ABCMeta):
     def __str__(self) -> str:
         return f"{self.id} ({DEVICE_TYPES.get(self.id[:2])})"
 
+    def __lt__(self, other) -> bool:
+        if not hasattr(other, "id"):
+            return NotImplemented
+        return self.id < other.id
+
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         # sometimes, battery-powered devices do respond to an RQ (e.g. bind mode)
         # super()._discover(discover_flag=discover_flag)
 
-        if discover_flag & DISCOVER_SCHEMA:
-            # self._send_cmd("1FC9", retries=0)
-            if self.type not in DEVICE_HAS_BATTERY and self.type not in ("13",):
+        if discover_flag & DISCOVER_SCHEMA and self.type not in DEVICE_HAS_BATTERY:
+            # self._send_cmd("1FC9", retries=3)
+            if self.type not in ("13",):
                 self._send_cmd("10E0", retries=0)  # TODO: use device hints
 
-        if discover_flag & DISCOVER_PARAMS:
-            pass
+        # if discover_flag & DISCOVER_PARAMS and self.type not in DEVICE_HAS_BATTERY:
+        #     pass
 
-        if discover_flag & DISCOVER_STATUS:
-            # self._send_cmd("0016", payload="0000", retries=0)
-            pass
+        # if discover_flag & DISCOVER_STATUS and self.type not in DEVICE_HAS_BATTERY:
+        #     self._send_cmd("0016", retries=3)
 
     def _send_cmd(self, code, **kwargs) -> None:
         dest = kwargs.pop("dest_addr", self.id)
@@ -173,7 +198,7 @@ class DeviceBase(Entity, metaclass=ABCMeta):
             _LOGGER.debug("Setting controller for %s to %s", self, ctl)
 
             # 064  I --- 01:078710 --:------ 01:144246 1F09 003 FF04B5  # has been seen
-            if not isinstance(ctl, Controller) and not ctl.is_controller:
+            if not isinstance(ctl, Controller) and not ctl._is_controller:
                 raise TypeError(f"Device {ctl} is not a controller")
 
             self._ctl = ctl
@@ -226,28 +251,27 @@ class Actuator:  # 3EF0, 3EF1
         super()._handle_msg(msg)
 
         if msg.code == "3EF0" and msg.verb == " I":  # NOT "RP", TODO: why????
-            self._actuator_state = msg
+            self._actuator_state = msg  # 3EF0
             self._actuator_enabled = msg
 
-            qos = {"priority": Priority.LOW, "retries": 1}
-            for code in ("0008", "3EF1"):
-                self._send_cmd(code, qos=qos)
+            self._send_cmd("3EF1", priority=Priority.LOW, retries=1)
 
         elif msg.code == "3EF1" and msg.verb == "RP":
-            self._actuator_cycle = msg
+            self._actuator_cycle = msg  # 3EF1
             self._actuator_enabled = msg
 
     @property
     def actuator_cycle(self) -> Optional[dict]:  # 3EF1
-        return _payload(self._actuator_cycle)
+        return self._msg_payload(self._actuator_cycle)
 
     @property
     def actuator_state(self) -> Optional[dict]:  # 3EF0
-        return _payload(self._actuator_state)
+        return self._msg_payload(self._actuator_state)
 
     @property
     def enabled(self) -> Optional[bool]:  # 3EF0, 3EF1
-        return _payload(self._actuator_enabled, "actuator_enabled")
+        """Return the actuator's current state."""
+        return self._msg_payload(self._actuator_enabled, "actuator_enabled")
 
     @property
     def status(self) -> dict:
@@ -255,7 +279,6 @@ class Actuator:  # 3EF0, 3EF1
             **super().status,
             "actuator_cycle": self.actuator_cycle,
             "actuator_state": self.actuator_state,
-            "enabled": self.enabled,
         }
 
 
@@ -273,11 +296,11 @@ class BatteryState:  # 1060
 
     @property
     def battery_low(self) -> Optional[bool]:  # 1060
-        return _payload(self._battery_state, "battery_low")
+        return self._msg_payload(self._battery_state, "battery_low")
 
     @property
     def battery_state(self) -> Optional[dict]:  # 1060
-        return _payload(self._battery_state)
+        return self._msg_payload(self._battery_state)
 
     @property
     def status(self) -> dict:
@@ -301,13 +324,13 @@ class Setpoint:  # 2309
 
     @property
     def setpoint(self) -> Optional[float]:  # 2309
-        return _payload(self._setpoint, "setpoint")
+        return self._msg_payload(self._setpoint, ATTR_SETPOINT)
 
     @property
     def status(self) -> dict:
         return {
             **super().status,
-            "setpoint": self.setpoint,
+            ATTR_SETPOINT: self.setpoint,
         }
 
 
@@ -325,13 +348,13 @@ class Temperature:  # 30C9
 
     @property
     def temperature(self) -> Optional[float]:  # 30C9
-        return _payload(self._temp, "temperature")
+        return self._msg_payload(self._temp, ATTR_TEMP)
 
     @property
     def status(self) -> dict:
         return {
             **super().status,
-            "temperature": self.temperature,
+            ATTR_TEMP: self.temperature,
         }
 
 
@@ -349,7 +372,7 @@ class DeviceInfo:  # 10E0
 
     @property
     def device_info(self) -> Optional[dict]:  # 10E0
-        return _payload(self._device_info)
+        return self._msg_payload(self._device_info)
 
     @property
     def schema(self) -> dict:
@@ -365,22 +388,6 @@ class Device(DeviceInfo, DeviceBase):
     def _handle_msg(self, msg) -> bool:
         super()._handle_msg(msg)
 
-        if msg.code in ("0016", "1FC9"):
-            pass
-
-        elif self.type not in HINTS_DEVICES:
-            assert False, f"Unknown device type: {self.id}"
-
-        elif msg.code not in HINTS_DEVICES[self.type]:
-            assert (
-                HINTS_DEVICES[self.type] == {}
-            ), f"Unknown code for {self.id}: {msg.verb}/{msg.code}"
-
-        elif msg.verb not in HINTS_DEVICES[self.type][msg.code]:
-            assert (
-                HINTS_DEVICES[self.type][msg.code] == []
-            ), f"Unknown verb for {self.id}: {msg.verb}/{msg.code}"
-
         # TODO: status updates always, but...
         # TODO: schema updates only if eavesdropping is enabled.
         # if self._ctl is not None and "parent_idx" in msg.payload:
@@ -450,7 +457,7 @@ class Device(DeviceInfo, DeviceBase):
         if self._zone is not None:
             if self._zone is not zone:
                 raise CorruptStateError(
-                    f"Device {self} has a mismatched parent zone: "
+                    f"Device {self} appears claimed by multiple parent zones: "
                     f"old={self._zone}, new={zone}"
                 )
             return
@@ -486,7 +493,7 @@ class Device(DeviceInfo, DeviceBase):
         return self._has_battery
 
     @property
-    def is_controller(self) -> Optional[bool]:  # 1F09
+    def _is_controller(self) -> Optional[bool]:  # 1F09
         if self._ctl is self:
             return True
         # if isinstance(self, Controller):
@@ -516,7 +523,7 @@ class Device(DeviceInfo, DeviceBase):
         return {}
 
 
-# 01:
+# CTL: 01:
 class Controller(Device):
     """The Controller base class."""
 
@@ -530,8 +537,17 @@ class Controller(Device):
         self.devices = list()  # [self]
         self.device_by_id = dict()  # {self.id: self}
 
+    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+        super()._discover(discover_flag=discover_flag)
 
-# 02: "10E0", "3150";; "0008", "22C9", "22D0"
+        if discover_flag & DISCOVER_SCHEMA and self.type not in DEVICE_HAS_BATTERY:
+            self._send_cmd("1FC9", retries=3)  # rf_bind
+
+        if discover_flag & DISCOVER_STATUS and self.type not in DEVICE_HAS_BATTERY:
+            self._send_cmd("0016", retries=3)  # rf_check
+
+
+# UFC: 02: "10E0", "3150";; "0008", "22C9", "22D0"
 class UfhController(Device):
     """The UFC class, the HCE80 that controls the UFH zones."""
 
@@ -543,6 +559,7 @@ class UfhController(Device):
 
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
+        self.klass = "UFC"
 
         self._circuits = {}
         self._setpoints = None
@@ -591,10 +608,12 @@ class UfhController(Device):
         super()._handle_msg(msg)
 
         if msg.code == "000C":
-            if "ufh_idx" in msg.payload and msg.payload["zone_id"] is not None:
-                self._circuits[msg.payload["ufh_idx"]] = {
-                    "zone_idx": msg.payload["zone_id"]
-                }
+            assert "ufh_idx" in msg.payload, "wsdfh"
+            if msg.payload["zone_id"] is not None:
+                self._circuits[msg.payload["ufh_idx"]] = msg
+
+        elif msg.code == "22C9":
+            self._setpoints = msg
 
         # "0008|FA/FC", "22C9|array", "22D0|none", "3150|ZZ/array(/FC?)"
 
@@ -603,32 +622,61 @@ class UfhController(Device):
         # else:
         #     assert False, "Unknown packet code"
 
+    @property
+    def circuits(self) -> Optional[Dict]:  # 000C
+        def fix(k):
+            return "zone_idx" if k == "zone_id" else k
+
+        return [
+            {fix(k): v for k, v in m.payload.items() if k in ("ufh_idx", "zone_id")}
+            for m in self._circuits.values()
+        ]
+
+    @property
+    def heat_demand(self) -> Optional[Dict]:  # 3150
+        try:
+            return self._msgs["3150"].payload[ATTR_HEAT_DEMAND]
+        except KeyError:
+            return
+
+    @property
+    def relay_demand(self) -> Optional[Dict]:  # 0008
+        try:
+            return self._msgs["0008"].payload["relay_demand"]
+        except KeyError:
+            return
+
     @property
     def setpoints(self) -> Optional[Dict]:  # 22C9
-        return _payload(self._setpoints)
+        return [
+            {k: v for k, v in d.items() if k in ("ufh_idx", "temp_high", "temp_low")}
+            for d in (self._setpoints.payload if self._setpoints else [])
+        ]
 
     @property  # id, type
     def schema(self) -> dict:
         return {
             **super().schema,
-            "ufh_circuits": self._circuits,
+            "circuits": self.circuits,
         }
 
-    # @property  # setpoint, config, mode (not schedule)
-    # def params(self) -> dict:
+    @property  # setpoint, config, mode (not schedule)
+    def params(self) -> dict:
+        return {
+            **super().params,
+            "setpoints": self.setpoints,
+        }
 
-    #     return {
-    #        **super().params,
-    #     }
-
-    # @property
-    # def status(self) -> dict:
-    #     return {
-    #         **super().status,
-    #     }
+    @property
+    def status(self) -> dict:
+        return {
+            **super().status,
+            ATTR_HEAT_DEMAND: self.heat_demand,
+            "relay_demand": self.relay_demand,
+        }
 
 
-# 07: "1260" "10A0" (and "1060")
+# DHW: 07: "1260" "10A0" (and "1060")
 class DhwSensor(BatteryState, Device):
     """The DHW class, such as a CS92."""
 
@@ -651,31 +699,35 @@ class DhwSensor(BatteryState, Device):
 
     # @property
     # def dhw_params(self) -> Optional[dict]:
-    #     return _payload(self._dhw_params)
+    #     return self._msg_payload(self._dhw_params)
 
     @property
     def temperature(self) -> Optional[float]:
-        return _payload(self._temp, "temperature")
+        return self._msg_payload(self._temp, ATTR_TEMP)
 
-    # @property
-    # def params(self) -> dict:
-    #     return {
-    #         **super().params,
-    #         "dhw_params": self.dhw_params,
-    #     }
+    @property
+    def params(self) -> dict:
+        return {
+            **super().params,
+            # "dhw_params": self.dhw_params,  # TODO
+        }
 
     @property
     def status(self) -> dict:
         return {
             **super().status,
-            "temperature": self.temperature,
+            ATTR_TEMP: self.temperature,
         }
 
 
-# 10: "10E0", "3EF0", "3150";; "22D9", "3220" ("1FD4"), TODO: 3220
+# OTB: 10: "10E0", "3EF0", "3150";; "22D9", "3220" ("1FD4"), TODO: 3220
 class OtbGateway(Actuator, Device):
     """The OTB class, specifically an OpenTherm Bridge (R8810A Bridge)."""
 
+    # see: https://www.opentherm.eu/request-details/?post_ids=2944
+
+    VALUE = "value"
+
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
@@ -686,48 +738,35 @@ class OtbGateway(Actuator, Device):
         self._opentherm_msg = {}
 
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        # 086 RQ --- 01:123456 63:262143 --:------ 3220 005 0000050000
-        # 049 RP --- 63:262143 01:123456 --:------ 3220 005 00F0050000
-        # 092 RQ --- 01:123456 63:262143 --:------ 3220 005 0000110000
-        # 049 RP --- 63:262143 01:123456 --:------ 3220 005 00C0110000
-        # 082 RQ --- 01:123456 63:262143 --:------ 3220 005 0000120000
-        # 076 RQ --- 01:123456 63:262143 --:------ 3220 005 0000120000
-        # 049 RP --- 63:262143 01:123456 --:------ 3220 005 0040120166
-        # 075 RQ --- 01:123456 63:262143 --:------ 3220 005 0080130000
-        # 049 RP --- 63:262143 01:123456 --:------ 3220 005 0070130000
-        # 074 RQ --- 01:123456 63:262143 --:------ 3220 005 0080190000
-        # 049 RP --- 63:262143 01:123456 --:------ 3220 005 00401929E6
-        # 072 RQ --- 01:123456 63:262143 --:------ 3220 005 00801A0000
-        # 048 RP --- 63:262143 01:123456 --:------ 3220 005 00401A3033
-        # 073 RQ --- 01:123456 63:262143 --:------ 3220 005 00801C0000
-        # 049 RP --- 63:262143 01:123456 --:------ 3220 005 00401C29B3
-        # 074 RQ --- 01:123456 63:262143 --:------ 3220 005 0080730000
-        # 048 RP --- 63:262143 01:123456 --:------ 3220 005 00407300CC
         super()._discover(discover_flag=discover_flag)
 
         if discover_flag & DISCOVER_SCHEMA:
-            # TODO: From OT v2.2: version numbers
-            for msg_id in range(124, 128):
-                self._send_cmd("3220", payload=f"0000{msg_id:02X}0000")
+            for msg_id in range(0x7C, 0x80):  # From OT v2.2: version numbers
+                self._gwy.send_cmd(Command.get_opentherm_msg(self.id, msg_id))
 
         if discover_flag & DISCOVER_PARAMS:
             pass
 
         if discover_flag & DISCOVER_STATUS:  # TODO: these need to be periodic
-            # From OT v2.2, these are mandatory: 00, 01, 03, 0E, 11, 19
-            # From evohome:
+            # From OT v2.2, these are mandatory: 00, 01, 03, 0E, 11, 19...
+            msg_ids = {0x00, 0x01, 0x03, 0x0E, 0x11, 0x19}
+
+            # and, From evohome: 05, 11, 12, 13, 19, 1A...
+            msg_ids |= {0x05, 0x11, 0x12, 0x13, 0x19, 0x1A}
             # 05 - Fault flags & OEM fault code
             # 11 - Relative modulation level
             # 12 - Central heating water pressure
             # 13 - DHW flow rate (litres/minute)
             # 19 - Boiler water temperature
             # 1A - DHW temperature
-            for msg_id in ("0005", "0011", "0012", "8013", "8019", "801A"):
-                self._send_cmd("3220", payload=f"00{msg_id}0000")
+
+            # and, others...
+            msg_ids |= {0x1B, 0x1C, 0x73}
+            # 1B - Outside temperature
             # 1C - Return water temperature
             # 73 - OEM diagnostic code
-            for msg_id in ("801C", "8073"):
-                self._send_cmd("3220", payload=f"00{msg_id}0000")
+            for msg_id in msg_ids:
+                self._gwy.send_cmd(Command.get_opentherm_msg(self.id, msg_id))
 
     def _handle_msg(self, msg) -> bool:
         super()._handle_msg(msg)
@@ -736,7 +775,7 @@ class OtbGateway(Actuator, Device):
             self._boiler_setpoint = msg
 
         elif msg.code == "3220" and msg.verb == "RP":  # TODO: what about I/W (or RQ)
-            self._opentherm_msg[msg.payload["id"]] = msg  # TODO: these need to expire
+            self._opentherm_msg[msg.payload["msg_id"]] = msg  # TODO: need to expire
 
         elif msg.code == "3EF0" and msg.verb == "RP":
             self._modulation_level = msg
@@ -745,52 +784,72 @@ class OtbGateway(Actuator, Device):
             self._modulation_level = msg
 
     @property
-    def boiler_temp(self) -> Optional[float]:  # 3220
-        if "19" in self._opentherm_msg:
-            return self._opentherm_msg["19"].payload["value"]
+    def boiler_water_temp(self) -> Optional[float]:  # 3220
+        try:
+            return self._opentherm_msg["0x19"].payload[self.VALUE]
+        except KeyError:
+            return
 
     @property
-    def ch_pressure(self) -> Optional[float]:  # 3220
-        if "12" in self._opentherm_msg:
-            return self._opentherm_msg["12"].payload["value"]
+    def ch_water_pressure(self) -> Optional[float]:  # 3220
+        try:
+            return self._opentherm_msg["0x12"].payload[self.VALUE]
+        except KeyError:
+            return
 
     @property
     def dhw_flow_rate(self) -> Optional[float]:  # 3220
-        if "13" in self._opentherm_msg:
-            return self._opentherm_msg["13"].payload["value"]
+        try:
+            return self._opentherm_msg["0x13"].payload[self.VALUE]
+        except KeyError:
+            return
 
     @property
-    def dwh_temp(self) -> Optional[float]:  # 3220
-        if "1A" in self._opentherm_msg:
-            return self._opentherm_msg["1A"].payload["value"]
+    def dhw_temp(self) -> Optional[float]:  # 3220
+        try:
+            return self._opentherm_msg["0x1A"].payload[self.VALUE]
+        except KeyError:
+            return
 
     @property
     def rel_modulation_level(self) -> Optional[float]:  # 3220
-        if "11" in self._opentherm_msg:
-            return self._opentherm_msg["11"].payload["value"]
+        try:
+            return self._opentherm_msg["0x11"].payload[self.VALUE]
+        except KeyError:
+            return
 
     @property
     def return_cv_temp(self) -> Optional[float]:  # 3220
-        if "1C" in self._opentherm_msg:
-            return self._opentherm_msg["1C"].payload["value"]
+        try:
+            return self._opentherm_msg["0x1C"].payload[self.VALUE]
+        except KeyError:
+            return
 
     @property
     def boiler_setpoint(self) -> Optional[float]:  # 22D9
-        return _payload(self._boiler_setpoint, "boiler_setpoint")
+        return self._msg_payload(self._boiler_setpoint, "boiler_setpoint")
 
     @property
     def modulation_level(self) -> Optional[float]:  # 3EF0/3EF1
-        return _payload(self._modulation_level, "modulation_level")
+        return self._msg_payload(self._modulation_level, "modulation_level")
 
     @property
-    def ot_status(self) -> dict:
+    def opentherm_status(self) -> dict:
         return {
-            slugify(self._opentherm_msg[msg_id].payload["msg_name"]): (
-                self._opentherm_msg[msg_id].payload["value"]
-            )
-            for msg_id in ("11", "12", "13", "19", "1A", "1C")
-            if msg_id in self._opentherm_msg
+            "boiler_temp": self.boiler_water_temp,
+            "ch_pressure": self.ch_water_pressure,
+            "dhw_flow_rate": self.dhw_flow_rate,
+            "dhw_temp": self.dhw_temp,
+            "rel_modulation_level": self.rel_modulation_level,
+            "return_cv_temp": self.return_cv_temp,
         }
+        # return {
+        #     slugify(self._opentherm_msg[msg_id].payload["msg_name"]): (
+        #         self._opentherm_msg[msg_id].payload["value"]
+        #     )
+        #     for msg_id in ("0x11", "0x12", "0x13", "0x19", "0x1A", "0x1C")
+        #     if msg_id in self._opentherm_msg
+        # }
 
     @property
     def status(self) -> dict:
@@ -798,16 +857,11 @@ class OtbGateway(Actuator, Device):
             **super().status,
             "boiler_setpoint": self.boiler_setpoint,
             "modulation_level": self.modulation_level,  # TODO: keep? (is duplicate)
-            "boiler_temp": self.boiler_temp,
-            "ch_pressure": self.ch_pressure,
-            "dhw_flow_rate": self.dhw_flow_rate,
-            "dwh_temp": self.dwh_temp,
-            "rel_modulation_level": self.rel_modulation_level,
-            "return_cv_temp": self.return_cv_temp,
+            "opentherm_status": self.opentherm_status,
         }
 
 
-# 03/12/22/34: 1060/2309/30C9;; (03/22: 0008/0009/3EF1, 2349?) (34: 000A/10E0/3120)
+# STA: 03/12/22/34: 1060/2309/30C9;; (03/22: 0008/0009/3EF1, 2349?) (34: 000A/10E0/3120)
 class Thermostat(BatteryState, Setpoint, Temperature, Device):
     """The THM/STA class, such as a TR87RF."""
 
@@ -815,7 +869,7 @@ class Thermostat(BatteryState, Setpoint, Temperature, Device):
         super()._handle_msg(msg)
 
 
-# 13: 0008/1100/3B00/3EF0/3EF1
+# BDR: 13: 0008/1100/3B00/3EF0/3EF1
 class BdrSwitch(Actuator, Device):
     """The BDR class, such as a BDR91.
 
@@ -838,12 +892,17 @@ class BdrSwitch(Actuator, Device):
         #     self._ctl._set_htg_control(self)
 
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        """The BDRs fail(?) to respond to RQs for: 3B00, 3EF0, 0009.
+        """The BDRs have one of six roles:
+         - heater relay *or* a heat pump relay (alternative to an OTB)
+         - DHW hot water valve *or* DHW heating valve
+         - Zones: Electric relay *or* Zone valve relay
 
-        They all seem to respond to (haven't checked a zone-valve zone):
-        - 0008: varies on/off
-        - 1100
-        - 3EF1: has sub-domains?
+        They all seem to respond thus (TODO heat pump/zone valve relay):
+         - all BDR91As will (erractically) RP to these RQs
+             0016, 1FC9 & 0008, 1100, 3EF1
+         - all BDR91As will *not* RP to these RQs
+             0009, 10E0, 3B00, 3EF0
+         - a BDR91A will *periodically* send an I/3B00/00CB if it is the heater relay
         """
 
         super()._discover(discover_flag=discover_flag)
@@ -852,7 +911,7 @@ class BdrSwitch(Actuator, Device):
             pass
 
         if discover_flag & DISCOVER_PARAMS:
-            self._send_cmd("1100")  # heater_relay wont RP to RQ/3B00?
+            self._send_cmd("1100")
 
         if discover_flag & DISCOVER_STATUS:
             for code in ("0008", "3EF1"):
@@ -872,6 +931,9 @@ class BdrSwitch(Actuator, Device):
             # for code in ("0008", "3EF1"):
             #     self._send_cmd(code, delay=1)
 
+        elif msg.code == "3EF0" and msg.verb == " I":  # NOT "RP", TODO: why????
+            self._send_cmd("0008", priority=Priority.LOW, retries=1)
+
     @property
     def role(self) -> Optional[str]:
         """Return the role of the BDR91A (there are six possibilities)."""
@@ -897,17 +959,17 @@ class BdrSwitch(Actuator, Device):
 
         if self._is_tpi:
             self._domain_id = "FC"  # TODO: check is None first
-            self.ctl._set_htg_control(self)
+            self._ctl._set_htg_control(self)
 
         return self._is_tpi
 
     @property
     def relay_demand(self) -> Optional[float]:  # 0008
-        return _payload(self._relay_demand, "relay_demand")
+        return self._msg_payload(self._relay_demand, "relay_demand")
 
     @property
     def tpi_params_wip(self) -> Optional[dict]:  # 1100
-        return _payload(self._tpi_params)
+        return self._msg_payload(self._tpi_params)
 
     @property
     def params(self) -> dict:
@@ -924,6 +986,7 @@ class BdrSwitch(Actuator, Device):
         }
 
 
+# TRV: 00/04:
 class TrvActuator(BatteryState, Setpoint, Temperature, Device):
     """The TRV class, such as a HR92."""
 
@@ -944,14 +1007,14 @@ class TrvActuator(BatteryState, Setpoint, Temperature, Device):
 
     @property
     def heat_demand(self) -> Optional[float]:  # 3150
-        return _payload(self._heat_demand, "heat_demand")
+        return self._msg_payload(self._heat_demand, ATTR_HEAT_DEMAND)
 
     @property
-    def window_state(self) -> Optional[bool]:  # 12B0
-        return _payload(self._window_state, "window_open")
+    def window_open(self) -> Optional[bool]:  # 12B0
+        return self._msg_payload(self._window_state, ATTR_WINDOW_OPEN)
 
     @property
-    def enabled(self) -> Optional[bool]:  # 3EF0, 3EF1
+    def enabled(self) -> Optional[bool]:
         if self.heat_demand is not None:
             return bool(self.heat_demand)
 
@@ -959,23 +1022,78 @@ class TrvActuator(BatteryState, Setpoint, Temperature, Device):
     def status(self) -> dict:
         return {
             **super().status,
-            "enabled": self.enabled,
-            "heat_demand": self.heat_demand,
-            "window_state": self.window_state,
+            ATTR_HEAT_DEMAND: self.heat_demand,
+            ATTR_WINDOW_OPEN: self.window_open,
         }
 
 
+# FAN: 39:
+class FanSwitch(BatteryState, Device):
+    """The FAN (switch) class, such as a 4-way switch."""
+
+    DEVICE_CLASS = "SWI"
+    DEVICE_TYPES = ("39",)
+
+    BOOST_TIMER = "boost_timer"  # e.g. 10, 20, 30 minutes
+    HEATER_MODE = "heater_mode"  # e.g. auto, off
+    HEATER_MODES = {9: "off", 10: "auto"}  # TODO:
+
+    FAN_MODE = "fan_mode"  # e.g. low. high
+    FAN_MODES = {
+        0: "standby",
+        1: "auto",
+        2: "low",
+        3: "medium",
+        4: "high",  # a.k.a. boost if timer on
+    }
+    FAN_RATE = "fan_rate"  # 0.0 - 1.0
+
+    @property
+    def fan_mode(self) -> Optional[float]:
+        return self._msg_payload(self._msgs["22F1"], self.FAN_MODE)
+
+    @property
+    def boost_timer(self) -> Optional[bool]:
+        return self._msg_payload(self._msgs["22F3"], self.BOOST_TIMER)
+
+    @property
+    def status(self) -> dict:
+        return {
+            **super().status,
+            self.FAN_MODE: self.fan_mode,
+            self.BOOST_TIMER: self.boost_timer,
+        }
+
+
+DEVICE_KLASS_TO_CLASS = {
+    "TRV": TrvActuator,
+    "CTL": Controller,
+    "PRG": Controller,
+    "UFC": UfhController,
+    "THM": Thermostat,
+    "DHW": DhwSensor,
+    "OTB": OtbGateway,
+    "BDR": BdrSwitch,
+    "SWI": FanSwitch,
+}
+DEVICE_TYPE_TO_KLASS = {
+    "00": "TRV",
+    "01": "CTL",
+    "02": "UFC",
+    "03": "THM",
+    "04": "TRV",
+    "07": "DHW",
+    "10": "OTB",
+    "12": "THM",
+    "13": "BDR",
+    "22": "THM",
+    "23": "PRG",
+    "34": "THM",
+    "39": "SWI",
+}
 DEVICE_CLASSES = {
-    "00": TrvActuator,
-    "01": Controller,
-    "02": UfhController,
-    "03": Thermostat,
-    "04": TrvActuator,
-    "07": DhwSensor,
-    "10": OtbGateway,
-    "12": Thermostat,
-    "13": BdrSwitch,
-    "22": Thermostat,
-    "23": Controller,
-    "34": Thermostat,
+    k1: v2
+    for k1, v1 in DEVICE_TYPE_TO_KLASS.items()
+    for k2, v2 in DEVICE_KLASS_TO_CLASS.items()
+    if v1 == k2
 }
