@@ -22,26 +22,29 @@ from serial import serial_for_url  # Serial, SerialException, serial_for_url
 from serial_asyncio import SerialTransport as SerialTransportAsync
 
 from .command import Command, Priority
-from .const import DTM_LONG_REGEX, HGI_DEVICE, NUL_DEVICE, _dev_mode_
+from .const import DTM_LONG_REGEX, HGI_DEV_ADDR, _dev_mode_
 from .helpers import dt_str
 from .packet import _PKT_LOGGER, Packet
 from .protocol import create_protocol_factory
-from .schema import DISABLE_SENDING, ENFORCE_ALLOWLIST, ENFORCE_BLOCKLIST, EVOFW_FLAG
+from .schema import (
+    ALLOW_LIST,
+    BLOCK_LIST,
+    DISABLE_SENDING,
+    ENFORCE_ALLOWLIST,
+    ENFORCE_BLOCKLIST,
+    EVOFW_FLAG,
+    SERIAL_CONFIG,
+    SERIAL_CONFIG_SCHEMA,
+)
 from .version import __version__
 
-DEV_MODE = _dev_mode_ or True
+DEV_MODE = _dev_mode_
 
 ERR_MSG_REGEX = re.compile(r"^([0-9A-F]{2}\.)+$")
 
 POLLER_TASK = "poller_task"
 
-SERIAL_CONFIG = {
-    "baudrate": 115200,
-    "timeout": 0,  # None
-    "dsrdtr": False,
-    "rtscts": False,
-    "xonxoff": True,  # set True to remove \x11
-}
+DEFAULT_SERIAL_CONFIG = SERIAL_CONFIG_SCHEMA(dict())
 
 Pause = SimpleNamespace(
     NONE=td(seconds=0),
@@ -51,9 +54,8 @@ Pause = SimpleNamespace(
     LONG=td(seconds=0.5),
 )
 
-INIT_QOS = {"priority": Priority.ASAP, "retries": 24, "disable_backoff": True}
-INIT_CMD = Command(" I", NUL_DEVICE.id, "0001", "00FFFF0200", qos=INIT_QOS)
-# INIT_CMD = Command(" I", HGI_DEVICE.id, "0001", "00FFFF0200", qos=INIT_QOS)
+INIT_QOS = {"priority": Priority.HIGHEST, "retries": 24, "disable_backoff": True}
+INIT_CMD = Command._puzzle(message=f"v{__version__}", **INIT_QOS)
 
 # tx (from sent to gwy, to get back from gwy) seems to takes approx. 0.025s
 QOS_TX_TIMEOUT = td(seconds=0.05)  # 0.20 OK, but too high?
@@ -63,17 +65,19 @@ QOS_RX_TIMEOUT = td(seconds=0.20)  # 0.10 too low sometimes
 QOS_MAX_BACKOFF = 3  # 4 = 16x, is too many?
 
 _LOGGER = logging.getLogger(__name__)
-if DEV_MODE:
-    _LOGGER.setLevel(logging.INFO)  # DEBUG may have too much detail
+_LOGGER.setLevel(logging.WARNING)  # INFO may have too much detail
+if DEV_MODE:  # or True:
+    _LOGGER.setLevel(logging.DEBUG)  # should be INFO
 
 
 class SerTransportFile(asyncio.Transport):
     """Interface for a packet transport using a file - Experimental."""
 
     def __init__(self, loop, protocol, packet_log, extra=None):
-        _LOGGER.debug("SerTransFile.__init__() *** PACKET_LOG VERSION ***")
+        _LOGGER.info("SerTransFile.__init__(%s) *** log file version ***", packet_log)
+
+        self._loop = loop
 
-        # self._loop = loop
         self._protocol = protocol
         self.fp = packet_log
         self._extra = {} if extra is None else extra
@@ -82,7 +86,8 @@ class SerTransportFile(asyncio.Transport):
 
     def _start(self):
         async def _polling_loop():
-            _LOGGER.debug("SerTransFile._polling_loop() BEGUN")
+            if DEV_MODE:
+                _LOGGER.debug("SerTransFile._polling_loop() BEGUN")
             self._protocol.pause_writing()
             self._protocol.connection_made(self)
 
@@ -90,12 +95,14 @@ class SerTransportFile(asyncio.Transport):
                 self._protocol.data_received(dtm_pkt_line.strip())
                 # await asyncio.sleep(0)
 
-            _LOGGER.debug("SerTransFile._polling_loop() ENDED")
+            if DEV_MODE:
+                _LOGGER.debug("SerTransFile._polling_loop() ENDED")
             self._protocol.connection_lost(exc=None)
 
-        _LOGGER.debug("SerTransFile._start()")
+        if DEV_MODE:
+            _LOGGER.debug("SerTransFile._start() STARTING loop")
 
-        self._extra[POLLER_TASK] = asyncio.create_task(_polling_loop())
+        self._extra[POLLER_TASK] = self._loop.create_task(_polling_loop())
 
     def write(self, cmd):
         """Write some data bytes to the transport."""
@@ -110,7 +117,7 @@ class SerTransportPoller(asyncio.Transport):
     MAX_BUFFER_SIZE = 500
 
     def __init__(self, loop, protocol, ser_instance, extra=None):
-        _LOGGER.warning("SerTransPoll.__init__() *** POLLING VERSION ***")
+        _LOGGER.info("SerTransPoll.__init__(%s) *** Polling version ***", ser_instance)
 
         self._loop = loop
         self._protocol = protocol
@@ -124,7 +131,8 @@ class SerTransportPoller(asyncio.Transport):
 
     def _start(self):
         async def _polling_loop():
-            _LOGGER.debug("SerTransPoll._polling_loop() BEGUN")
+            if DEV_MODE:
+                _LOGGER.debug("SerTransPoll._polling_loop() BEGUN")
             self._protocol.connection_made(self)
 
             while self.serial.is_open:
@@ -144,13 +152,15 @@ class SerTransportPoller(asyncio.Transport):
                     self._write_queue.task_done()
                     continue
 
-            _LOGGER.error("SerTransPoll._polling_loop() ENDED")
+            if DEV_MODE:
+                _LOGGER.error("SerTransPoll._polling_loop() ENDED")
             self._protocol.connection_lost()
 
-        _LOGGER.debug("SerTransPoll._start()")
+        if DEV_MODE:
+            _LOGGER.debug("SerTransPoll._start() STARTING loop")
         self._write_queue = Queue(maxsize=self.MAX_BUFFER_SIZE)
 
-        self._extra[POLLER_TASK] = asyncio.create_task(_polling_loop())
+        self._extra[POLLER_TASK] = self._loop.create_task(_polling_loop())
 
     def write(self, cmd):
         """Write some data bytes to the transport.
@@ -167,9 +177,10 @@ class SerTransportProcess(Process):  # TODO: WIP
     """Interface for a packet transport using a process - WIP."""
 
     def __init__(self, loop, protocol, ser_port, extra=None):
-        _LOGGER.warning("SerTransProc.__init__() *** PROCESS VERSION***")
+        _LOGGER.info("SerTransProc.__init__() *** Process version ***", ser_port)
 
         self._loop = loop
+
         self._protocol = protocol
         self._ser_port = ser_port
         self._extra = {} if extra is None else extra
@@ -183,7 +194,8 @@ class SerTransportProcess(Process):  # TODO: WIP
 
     def _start(self):
         def _polling_loop(self):
-            _LOGGER.error("WinTransport._polling_loop()")
+            if DEV_MODE:
+                _LOGGER.error("WinTransport._polling_loop() BEGUN")
 
             # asyncio.set_event_loop(self._loop)
             asyncio.get_running_loop()  # TODO: this fails
@@ -214,7 +226,12 @@ class SerTransportProcess(Process):  # TODO: WIP
                 # print("sleep")
                 # time.sleep(0.005)
 
-        _LOGGER.debug("SerTransProc.start()")
+            if DEV_MODE:
+                _LOGGER.debug("SerTransProc._polling_loop() ENDED")
+            self._protocol.connection_lost(exc=None)
+
+        if DEV_MODE:
+            _LOGGER.debug("SerTransProc._start() STARTING loop")
         self._write_queue = Queue(maxsize=200)
 
         self.serial = serial_for_url(self._ser_port[0], **self._ser_port[1])
@@ -236,53 +253,54 @@ class SerTransportProcess(Process):  # TODO: WIP
         self._write_queue.put_nowait(cmd)
 
 
-class PacketProtocol(asyncio.Protocol):
+class PacketProtocolBase(asyncio.Protocol):
     """Interface for a packet protocol (no Qos).
 
     ex transport: self.data_received(bytes) -> self._callback(pkt)
     to transport: self.send_data(cmd)       -> self._transport.write(bytes)
     """
 
-    def __init__(self, gwy, pkt_receiver: Callable) -> None:
-        _LOGGER.debug("PktProtocol.__init__(%s, %s)", gwy, pkt_receiver)
+    def __init__(self, gwy, pkt_handler: Callable) -> None:
+        self._loop = gwy._loop
 
         self._gwy = gwy
-        self._callback = pkt_receiver
+        self._callback = pkt_handler  # Could be None
 
         self._transport = None
         self._pause_writing = True
         self._recv_buffer = bytes()
 
-        # TODO: this is a little messy...
+        self._sequence_no = 0
+
+        # TODO: this is a little messy - the gwy.config[...] are mutex...
         self._include = list(gwy._include) if gwy.config[ENFORCE_ALLOWLIST] else []
         self._exclude = list(gwy._exclude) if gwy.config[ENFORCE_BLOCKLIST] else []
 
+        if self._include:
+            _LOGGER.info("Using an %s: %s", ALLOW_LIST, self._include)
+        elif self._exclude:
+            _LOGGER.info("Using an %s: %s", BLOCK_LIST, self._exclude)
+        else:
+            _LOGGER.warning("Not Using an device filter (an allow_list is recommended)")
+
         self._has_initialized = None
         if not self._gwy.config[DISABLE_SENDING]:
-            asyncio.create_task(self.send_data(INIT_CMD))  # HACK: port wakeup
+            self._loop.create_task(self.send_data(INIT_CMD))  # HACK: port wakeup
 
     def connection_made(self, transport: asyncio.Transport) -> None:
         """Called when a connection is made."""
         _LOGGER.debug("PktProtocol.connection_made(%s)", transport)
 
-        # print(transport.serial)  # TODO: evofw_flag here
-        # for attr in dir(transport.serial):
-        #     print("obj.%s = %r" % (attr, getattr(transport, attr)))
-
-        # from time import sleep
-        # sleep(4)
-
-        # print(transport.serial)  # TODO: evofw_flag here
-        # for attr in dir(transport.serial):
-        #     print("obj.%s = %r" % (attr, getattr(transport, attr)))
+        self._transport = transport
+        # self._transport.serial.rts = False
 
         _PKT_LOGGER.warning(
             "# evohome_rf %s", __version__, extra=self._extra(dt_str(), "")
         )
 
-        self._transport = transport
-        # self._transport.serial.rts = False
-
+        self._loop.create_task(
+            self._send_data(bytes("!V\r\n".encode("ascii")), ignore_pause=False)
+        )  # Used to see if using a evofw3 rather than a HGI80
         self._pause_writing = False  # TODO: needs work
 
     @staticmethod
@@ -307,25 +325,29 @@ class PacketProtocol(asyncio.Protocol):
         # 095  I --- 18:013393 18:000730 --:------ 0001 005 00FFFF0200 # HGI80
         # 000  I --- 18:140805 18:140805 --:------ 0001 005 00FFFF0200 # evofw3
         if pkt_line[10:14] == " 18:" and pkt_line[11:20] == pkt_line[21:30]:
-            pkt_line = pkt_line[:21] + HGI_DEVICE.id + pkt_line[30:]
-            _LOGGER.debug("evofw3 packet line has been normalised (0x00)")
+            pkt_line = pkt_line[:21] + HGI_DEV_ADDR.id + pkt_line[30:]
+            if DEV_MODE:  # TODO: should be _LOGGER.debug
+                _LOGGER.warning("evofw3 packet line has been normalised (0x00)")
 
         # non-RAMSES-II packets...
         elif (
             pkt_line[10:14] in (" 08:", " 31:") and pkt_line[-16:] == "* Checksum error"
         ):
             pkt_line = pkt_line[:-17] + " # Checksum error (ignored)"
-            # _LOGGER.debug("Packet line has been normalised (0x01)")
+            if DEV_MODE:  # TODO: should be _LOGGER.debug
+                _LOGGER.warning("Packet line has been normalised (0x01)")
 
         # bug fixed in evofw3 v0.6.x...
         elif pkt_line.startswith("!C"):
             pkt_line = "# " + pkt_line
-            # _LOGGER.debug("Packet line has been normalised (0x02)")
+            if DEV_MODE:  # TODO: should be _LOGGER.debug
+                _LOGGER.warning("Packet line has been normalised (0x02)")
 
         # old packet logs
         elif ERR_MSG_REGEX.match(pkt_line):
             pkt_line = "# " + pkt_line
-            # _LOGGER.debug("Packet line has been normalised (0x03)")
+            if DEV_MODE:  # TODO: should be _LOGGER.debug
+                _LOGGER.warning("Packet line has been normalised (0x03)")
 
         return pkt_line
 
@@ -333,6 +355,7 @@ class PacketProtocol(asyncio.Protocol):
         self, pkt_dtm: str, pkt_str: Optional[str], pkt_raw: Optional[ByteString] = None
     ) -> None:
         """Called when some normalised data is received (no QoS)."""
+        _LOGGER.info("PktProtocol.data_rcvd(%s)", pkt_raw)
 
         pkt = Packet(pkt_dtm, pkt_str, raw_pkt_line=pkt_raw)
         if not pkt.is_valid:
@@ -340,12 +363,13 @@ class PacketProtocol(asyncio.Protocol):
         elif self._has_initialized is None:
             self._has_initialized = True
 
-        if self.is_wanted(pkt, self._include, self._exclude):
+        if self._callback and self.is_wanted(pkt, self._include, self._exclude):
             self._callback(pkt)  # only wanted PKTs up to the MSG transport's handler
 
     def data_received(self, data: ByteString) -> None:
-        """Called when some data is received."""
-        _LOGGER.debug("PktProtocol.data_received(%s)", data)
+        """Called when some data (raw packet fragment) is received."""
+        if DEV_MODE:
+            _LOGGER.debug("PktProtocol.data_rcvd(%s)", data)
 
         def create_pkt(pkt_raw: ByteString) -> Tuple:
             dtm_str = dt_str()  # done here & now for most-accurate timestamp
@@ -369,9 +393,10 @@ class PacketProtocol(asyncio.Protocol):
             ):
                 flag = self._gwy.config[EVOFW_FLAG]
                 data = bytes(f"{flag}\r\n".encode("ascii"))
-                asyncio.create_task(self._send_data(data, ignore_pause=True))
+                self._loop.create_task(self._send_data(data, ignore_pause=True))
 
-            _PKT_LOGGER.debug("Rx: %s", pkt_raw, extra=self._extra(dtm_str, pkt_raw))
+            if DEV_MODE:  # TODO: deleteme
+                _LOGGER.debug("Rx: %s", pkt_raw, extra=self._extra(dtm_str, pkt_raw))
 
             return dtm_str, self._normalise(pkt_str), pkt_raw
 
@@ -388,18 +413,25 @@ class PacketProtocol(asyncio.Protocol):
 
         The _pause_writing flag can be ignored, is useful for sending traceflags.
         """
+
         if not ignore_pause:
             while self._pause_writing:
                 await asyncio.sleep(0.005)
-        while self._transport is None or self._transport.serial.out_waiting:
+        while (
+            self._transport is None
+            or self._transport.serial is None  # Shouldn't be required, but is!
+            or self._transport.serial.out_waiting
+        ):
             await asyncio.sleep(0.005)
-        _PKT_LOGGER.debug("Tx:     %s", data, extra=self._extra(dt_str(), data))
+        if DEV_MODE:  # TODO: deleteme
+            _LOGGER.debug("Tx:     %s", data, extra=self._extra(dt_str(), data))
+
         self._transport.write(data)
         # await asyncio.sleep(0.05)
 
     async def send_data(self, cmd: Command) -> None:
         """Called when some data is to be sent (not a callback)."""
-        _LOGGER.debug("PktProtocol.send_data(%s)", cmd)
+        _LOGGER.info("PktProtocol.send_data(%s)", cmd)
 
         if self._gwy.config[DISABLE_SENDING]:
             raise RuntimeError("Sending is disabled")
@@ -410,6 +442,18 @@ class PacketProtocol(asyncio.Protocol):
             )
             return
 
+        self._sequence_no = (self._sequence_no + 1) % 256
+        if False and self._qos_cmd.seqn == "---":
+            self._qos_cmd.seqn = f"{self._sequence_no:03d}"
+
+        if cmd.from_addr.type != "18":
+            _LOGGER.warning("PktProtocol.send_data(%s): IMPERSONATING!", cmd.tx_header)
+            kmd = Command._puzzle("02", cmd.tx_header)
+            await self._send_data(bytes(f"{kmd}\r\n".encode("ascii")))
+
+        # self._loop.create_task(
+        #     self._send_data(bytes(f"{cmd}\r\n".encode("ascii")))
+        # )
         await self._send_data(bytes(f"{cmd}\r\n".encode("ascii")))
 
     def connection_lost(self, exc: Optional[Exception]) -> None:
@@ -421,14 +465,14 @@ class PacketProtocol(asyncio.Protocol):
 
     def pause_writing(self) -> None:
         """Called when the transport's buffer goes over the high-water mark."""
-        _LOGGER.debug("PktProtocol.pause_writing()")
+        _LOGGER.warning("PktProtocol.pause_writing()")
         # self._transport.get_write_buffer_size()
 
         self._pause_writing = True
 
     def resume_writing(self) -> None:
         """Called when the transport's buffer drains below the low-water mark."""
-        _LOGGER.debug("PktProtocol.resume_writing()")
+        _LOGGER.warning("PktProtocol.resume_writing()")
         # self._transport.get_write_buffer_size()
 
         self._pause_writing = False
@@ -446,12 +490,40 @@ class PacketProtocol(asyncio.Protocol):
         }
 
 
-class PacketProtocolFile(PacketProtocol):
+class PacketProtocol(PacketProtocolBase):
+    """Interface for a packet protocol (without QoS)."""
+
+    def __init__(self, gwy, pkt_handler: Callable) -> None:
+        _LOGGER.info(
+            "PktProtocol.__init__(gwy, %s) *** Std version ***",
+            pkt_handler.__name__ if pkt_handler else None,
+        )
+        super().__init__(gwy, pkt_handler)
+
+
+class PacketProtocolFile(PacketProtocolBase):
     """Interface for a packet protocol (for packet log)."""
 
+    def __init__(self, gwy, pkt_handler: Callable) -> None:
+        _LOGGER.info(
+            "PacketProtocolFile.__init__(gwy, %s) *** Log version ***",
+            pkt_handler.__name__ if pkt_handler else None,
+        )
+        super().__init__(gwy, pkt_handler)
+
+    def connection_made(self, transport: asyncio.Transport) -> None:
+        """Called when a connection is made."""
+        _LOGGER.debug("PacketProtocolFile.connection_made(%s)", transport)
+
+        self._transport = transport
+
+        _PKT_LOGGER.warning(
+            "# evohome_rf %s", __version__, extra=self._extra(dt_str(), "")
+        )
+
     def data_received(self, data: str) -> None:
         """Called when some data is received."""
-        _LOGGER.debug("PktProtocolFile.data_received(%s)", data)
+        _LOGGER.debug("PktProtocolFile.data_rcvd(%s)", data)
 
         pkt_dtm, pkt_str = data[:26], data[27:]
 
@@ -471,10 +543,14 @@ class PacketProtocolFile(PacketProtocol):
             self._data_received(pkt_dtm, self._normalise(pkt_str), None)
 
 
-class PacketProtocolQos(PacketProtocol):
+class PacketProtocolQos(PacketProtocolBase):
     """Interface for a packet protocol (includes QoS)."""
 
     def __init__(self, gwy, pkt_handler: Callable) -> None:
+        _LOGGER.info(
+            "PktProtocol.__init__(gwy, %s) *** Qos version ***",
+            pkt_handler.__name__ if pkt_handler else None,
+        )
         super().__init__(gwy, pkt_handler)
 
         self._qos_lock = Lock()
@@ -508,6 +584,7 @@ class PacketProtocolQos(PacketProtocol):
         self, pkt_dtm: str, pkt_str: Optional[str], pkt_raw: Optional[ByteString] = None
     ) -> None:
         """Called when some data is received. Adjust backoff as required."""
+        _LOGGER.info("PktProtocolQos.data_rcvd(%s)", pkt_raw)
 
         def _logger_rcvd(logger, msg: str) -> None:
             if self._qos_cmd is None:
@@ -518,7 +595,7 @@ class PacketProtocolQos(PacketProtocol):
                 wanted = self._rx_hdr
 
             logger(
-                "PktProtocol.data_rcvd(%s): boff=%s, want=%s, tout=%s: %s",
+                "PktProtocolQos.data_rcvd(%s): boff=%s, want=%s, tout=%s: %s",
                 pkt._header,
                 self._backoff,
                 wanted,
@@ -535,46 +612,64 @@ class PacketProtocolQos(PacketProtocol):
         if self._qos_cmd:
             # _logger_rcvd(_LOGGER.debug, "CHECKING")
 
-            if pkt._header == self._tx_hdr and self._rx_hdr is None:  # echo of tx pkt
-                msg = "matched Tx (now done)"  # no response is expected
+            # NOTE: is the Tx pkt, and no response is expected
+            if pkt._header == self._tx_hdr and self._rx_hdr is None:
+                msg = "matched the Tx pkt (not wanting a Rx pkt) - now done"
                 self._qos_lock.acquire()
                 self._qos_cmd = None
                 self._qos_lock.release()
 
-            elif pkt._header == self._tx_hdr:  # echo of (expected) tx pkt
-                msg = "matched Tx (now wanting Rx)"
+            # NOTE: is the Tx pkt, and a response *is* expected
+            elif pkt._header == self._tx_hdr:
+                # assert str(pkt)[4:] == str(self._qos_cmd), "Packets dont match"
+                msg = "matched the Tx pkt (now wanting a Rx pkt)"
                 self._tx_hdr = None
 
-            elif pkt._header == self._rx_hdr:  # rcpt of (expected) rx pkt
-                msg = "matched Rx (now done)"
+            # NOTE: is the Tx pkt, but is a *duplicate* - we've already seen it!
+            elif pkt._header == self._qos_cmd.tx_header:
+                # assert str(pkt) == str(self._qos_cmd), "Packets dont match"
+                msg = "duplicated Tx pkt (still wanting the Rx pkt)"
+                self._timeouts(dt.now())  # TODO: increase backoff?
+
+            # NOTE: is the Rx pkt, and is a non-Null (expected) response
+            elif pkt._header == self._rx_hdr:
+                msg = "matched the Rx pkt - now done"
                 self._qos_lock.acquire()
                 self._qos_cmd = None
                 self._qos_lock.release()
 
-            elif pkt._header == self._qos_cmd.tx_header:  # rcpt of (duplicate!) rx pkt
-                msg = "duplicated Tx (still wanting Rx)"
-                self._timeouts(dt.now())  # TODO: increase backoff?
+            # TODO: is the Rx pkt, but is a Null response
+            # elif pkt._header == self._qos_cmd.null_header:
+            #     msg = "matched a NULL Rx pkt - now done"
+            #     self._qos_lock.acquire()
+            #     self._qos_cmd = None
+            #     self._qos_lock.release()
 
-            else:  # not the packet that was expected
-                msg = "unmatched (still wanting " + ("Tx)" if self._tx_hdr else "Rx)")
+            # NOTE: is not the expected pkt, but another pkt
+            else:
+                msg = (
+                    "unmatched pkt (still wanting a "
+                    + ("Tx" if self._tx_hdr else "Rx")
+                    + " pkt)"
+                )
 
             self._timeouts(dt.now())
             _logger_rcvd(_LOGGER.debug, f"CHECKED - {msg}")
 
-        else:  # throttle down the backoff
-            # self._timeouts(dt.now())
-            _logger_rcvd(_LOGGER.debug, "XXXXXXX - ")
+        # else:  # no outstanding cmd - ?throttle down the backoff
+        #     # self._timeouts(dt.now())
+        #     _logger_rcvd(_LOGGER.debug, "XXXXXXX - ")
 
-        if self.is_wanted(pkt, self._include, self._exclude):
+        if self._callback and self.is_wanted(pkt, self._include, self._exclude):
             self._callback(pkt)  # only wanted PKTs up to the MSG transport's handler
 
     async def send_data(self, cmd: Command) -> None:
         """Called when some data is to be sent (not a callback)."""
-        _LOGGER.debug("PktProtocolQos.send_data(%s)", cmd)
+        _LOGGER.info("PktProtocolQos.send_data(%s)", cmd)
 
         def _logger_send(logger, msg: str) -> None:
             logger(
-                "PktProtocol.send_data(%s): boff=%s, want=%s, tout=%s: %s",
+                "PktProtocolQos.send_data(%s): boff=%s, want=%s, tout=%s: %s",
                 cmd.tx_header,
                 self._backoff,
                 self._tx_hdr if self._tx_hdr else self._rx_hdr,
@@ -587,12 +682,10 @@ class PacketProtocolQos(PacketProtocol):
 
         if not cmd.is_valid:
             _LOGGER.warning(
-                "PktProtocol.send_data(%s): invalid command: %s", cmd.tx_header, cmd
+                "PktProtocolQos.send_data(%s): invalid command: %s", cmd.tx_header, cmd
             )
             return
 
-        # _logger_send(_LOGGER.debug, "SENDING")
-
         while self._qos_cmd is not None:
             await asyncio.sleep(0.005)
 
@@ -604,6 +697,17 @@ class PacketProtocolQos(PacketProtocol):
         self._tx_retries = 0
         self._tx_retry_limit = cmd.qos.get("retries", QOS_TX_RETRIES)
 
+        self._sequence_no = (self._sequence_no + 1) % 256
+        if False and self._qos_cmd.seqn == "---":
+            self._qos_cmd.seqn = f"{self._sequence_no:03d}"
+
+        if cmd.from_addr.type != "18":
+            _LOGGER.warning(
+                "PacketProtocolQos.send_data(%s): IMPERSONATING!", cmd.tx_header
+            )
+            kmd = Command._puzzle("02", cmd.tx_header)
+            await self._send_data(bytes(f"{kmd}\r\n".encode("ascii")))
+
         self._timeouts(dt.now())
         await self._send_data(bytes(f"{cmd}\r\n".encode("ascii")))
 
@@ -623,8 +727,8 @@ class PacketProtocolQos(PacketProtocol):
                 await self._send_data(bytes(f"{cmd}\r\n".encode("ascii")))
                 _logger_send(
                     _LOGGER.warning,
-                    f"RE-SENT ({self._tx_retries}/{self._tx_retry_limit})"
-                )
+                    f"RE-SENT ({self._tx_retries}/{self._tx_retry_limit})",
+                )  # TODO: should be info/debug
 
             else:
                 if self._qos_cmd.code != "7FFF":  # HACK: why expired when shouldn't
@@ -638,7 +742,7 @@ class PacketProtocolQos(PacketProtocol):
         else:
             if self._timeout_half >= dt.now():
                 self._backoff = max(self._backoff - 1, 0)
-            # _logger_send(_LOGGER.debug, "SUCCEEDED")
+            # _logger_send(_LOGGER.debug, "SENT OK")
 
 
 def create_pkt_stack(
@@ -671,11 +775,14 @@ def create_pkt_stack(
         pkt_transport = SerTransportFile(gwy._loop, pkt_protocol, packet_log)
         return (pkt_protocol, pkt_transport)
 
-    ser_instance = serial_for_url(serial_port, **SERIAL_CONFIG)
-    if os.name == "posix":
+    serial_config = DEFAULT_SERIAL_CONFIG
+    serial_config.update(gwy.config[SERIAL_CONFIG])
+
+    ser_instance = serial_for_url(serial_port, **serial_config)
+    if os.name == "posix":  # or use: NotImplementedError
         try:
             ser_instance.set_low_latency_mode(True)  # only for FTDI?
-        except ValueError:
+        except (AttributeError, ValueError):  # AttributeError shouldn't be needed
             pass
 
     if os.name == "nt":
