@@ -4,11 +4,12 @@
 """Evohome RF - The evohome-compatible zones."""
 
 from abc import ABCMeta, abstractmethod
-import asyncio
+from asyncio import Task
+from datetime import datetime as dt, timedelta as td
 import logging
 from typing import Optional
 
-from .command import Schedule
+from .command import Command, Schedule
 from .const import (
     ATTR_DEVICES,
     ATTR_DHW_SENSOR,
@@ -17,10 +18,9 @@ from .const import (
     ATTR_HEAT_DEMAND,
     ATTR_SETPOINT,
     ATTR_TEMP,
-    ATTR_OPEN_WINDOW,
+    ATTR_WINDOW_OPEN,
     ATTR_ZONE_SENSOR,
     ATTR_ZONE_TYPE,
-    # CODE_000C_DEVICE_TYPE,
     DEVICE_HAS_ZONE_SENSOR,
     DHW_STATE_MAP,
     DISCOVER_SCHEMA,
@@ -30,13 +30,12 @@ from .const import (
     ZONE_CLASS_MAP,
     ZONE_TYPE_MAP,
     ZONE_TYPE_SLUGS,
-    ZONE_MODE_LOOKUP,
-    ZONE_MODE_MAP,
     _dev_mode_,
 )
-from .devices import Device, Entity, _payload
+from .devices import Device, Entity
 from .exceptions import CorruptStateError
 from .helpers import dtm_to_hex
+from .ramses import RAMSES_ZONES, RAMSES_ZONES_ALL
 
 DEV_MODE = _dev_mode_
 
@@ -45,25 +44,16 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-def _temp(value) -> str:
-    """Return a two's complement Temperature/Setpoint."""
-    if value is None:
-        return "7FFF"
-
-    try:
-        _value = float(value)
-    except (ValueError, TypeError):
-        raise ValueError(f"Invalid temperature: {value}")
-
-    if _value < 0:
-        raise ValueError(f"Invalid temperature: {value}")
-
-    return f"{int(_value*100):04X}"
-
-
 class ZoneBase(Entity, metaclass=ABCMeta):
     """The Zone/DHW base class."""
 
+    DHW = "DHW"
+    ELE = "ELE"
+    MIX = "MIX"
+    RAD = "RAD"
+    UFH = "UFH"
+    VAL = "VAL"
+
     def __init__(self, evo, zone_idx) -> None:
         _LOGGER.debug("Creating a Zone: %s_%s (%s)", evo.id, zone_idx, self.__class__)
         super().__init__(evo._gwy)
@@ -77,22 +67,39 @@ class ZoneBase(Entity, metaclass=ABCMeta):
         self._evo = evo
         self._ctl = evo._ctl
 
-    # def __repr__(self) -> str:
-    #     return json.dumps(self.schema, indent=2)
+    def __repr__(self) -> str:
+        return f"{self.id} ({self.heating_type})"
 
-    def __str__(self) -> str:
-        return f"{self.id} ({self._zone_type})"
+    def __lt__(self, other) -> bool:
+        if not hasattr(other, "idx"):
+            return NotImplemented
+        return self.idx < other.idx
 
     @abstractmethod
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         raise NotImplementedError
 
-    def _handle_msg(self, msg) -> bool:
-        # super()._handle_msg(msg)
-        pass
+    def _handle_msg(self, msg) -> None:
+        """Validate packets by verb/code."""
+        super()._handle_msg(msg)
 
-        # else:
-        #     assert False, f"Unknown packet ({msg.verb}/{msg.code}) for {self.id}"
+        if True or not self._zone_type:
+            return
+
+        ramses_zones = RAMSES_ZONES if self._zone_type else {None: RAMSES_ZONES_ALL}
+
+        if self._zone_type not in ramses_zones:
+            assert False, f"Unknown zone type: {str(self)} (likely a corrupt pkt)"
+
+        elif msg.code not in ramses_zones[self._zone_type]:
+            assert (
+                ramses_zones[self._zone_type] == {}
+            ), f"Unknown code for {str(self)}: {msg.verb}/{msg.code}"
+
+        elif msg.verb not in ramses_zones[self._zone_type][msg.code]:
+            assert (
+                ramses_zones[self._zone_type][msg.code] == {}
+            ), f"Unknown verb for {str(self)}: {msg.verb}/{msg.code}"
 
     def _send_cmd(self, code, **kwargs) -> None:
         dest = kwargs.pop("dest_addr", self._ctl.id)
@@ -136,8 +143,14 @@ class ZoneBase(Entity, metaclass=ABCMeta):
 
     @property
     @abstractmethod
-    def mode(self) -> dict:
-        """Return the operating mode of the zone/DHW ({mode, setpoint, until})."""
+    def config(self) -> Optional[dict]:
+        """Return the configuration (parameters) of the zone/DHW."""
+        raise NotImplementedError
+
+    @property
+    @abstractmethod
+    def mode(self) -> Optional[dict]:
+        """Return the operating mode of the zone/DHW (mode, setpoint/active, until)."""
         raise NotImplementedError
 
     @property
@@ -155,10 +168,7 @@ class ZoneBase(Entity, metaclass=ABCMeta):
     @property
     @abstractmethod
     def temperature(self) -> Optional[float]:
-        """Return the measured temperature of the zone/DHW.
-
-        The controller, not the temperature sensor, is the source of this data.
-        """
+        """Return the measured temperature of the zone/DHW."""
         raise NotImplementedError
 
     # @property
@@ -172,32 +182,14 @@ class ZoneBase(Entity, metaclass=ABCMeta):
     #     """Set the type of the zone/DHW (e.g. electric_zone, stored_dhw)."""
     #     raise NotImplementedError
 
-
-class HeatDemand:  # 3150
-    def __init__(self, *args, **kwargs) -> None:
-        super().__init__(*args, **kwargs)
-
-        self._heat_demand = None
-
-    def _handle_msg(self, msg) -> bool:
-        super()._handle_msg(msg)
-
-        if msg.code == "3150" and msg.verb == " I":
-            self._heat_demand = msg
-
     @property
-    def heat_demand(self) -> Optional[float]:  # 3150
-        return _payload(self._heat_demand, "heat_demand")
-
-    @property
-    def status(self) -> dict:
-        return {
-            **super().status,
-            "heat_demand": self.heat_demand,
-        }
+    @abstractmethod
+    def heat_demand(self) -> Optional[float]:
+        """Return the (estimated) heat_demand of the zone/DHW."""
+        raise NotImplementedError
 
 
-class DhwZone(ZoneBase, HeatDemand):
+class DhwZone(ZoneBase):
     """The DHW class."""
 
     def __init__(self, ctl, sensor=None, dhw_valve=None, htg_valve=None) -> None:
@@ -205,17 +197,19 @@ class DhwZone(ZoneBase, HeatDemand):
 
         ctl._set_dhw(self)
 
-        self._sensor = None
-        # self._dhw_valve = None
-        # self._htg_valve = None
+        self._dhw_sensor = None
+        self._dhw_valve = None
+        self._htg_valve = None
 
-        self.heating_type = "DHW"
+        self.heating_type = Zone.DHW
 
         self._dhw_mode = None
         self._dhw_params = None
+        self._dhw_temp = None
         self._relay_demand = None
         self._relay_failsafe = None
-        self._temp = None
+
+        self._heat_demand = None
 
         if sensor:
             self._set_sensor(sensor)
@@ -257,105 +251,168 @@ class DhwZone(ZoneBase, HeatDemand):
             self._dhw_params = msg
 
         elif msg.code == "1260":
-            self._temp = msg
+            self._dhw_temp = msg
 
         elif msg.code == "1F41":
-            self._setpoint_status = msg
+            self._dhw_mode = msg
 
-        # elif msg.code in ("1100", "3150", "3B00"):
-        #     pass
-
-    @property
-    def sensor(self) -> Device:
-        return self._sensor
-
-    def _set_sensor(self, device: Device) -> None:  # self._sensor
+    def _set_sensor(self, device: Device) -> None:
         """Set the temp sensor for this DHW system (07: only)."""
 
-        if self._sensor != device and self._sensor is not None:
+        if self._dhw_sensor != device and self._dhw_sensor is not None:
             raise CorruptStateError(
-                f"{ATTR_ZONE_SENSOR} shouldn't change: {self._sensor} to {device}"
+                f"{ATTR_ZONE_SENSOR} shouldn't change: {self._dhw_sensor} to {device}"
             )
 
         if not isinstance(device, Device) or device.type != "07":
             raise TypeError(f"{ATTR_ZONE_SENSOR} can't be: {device}")
 
-        if self._sensor is None:
-            self._sensor = device
+        if self._dhw_sensor is None:
+            self._dhw_sensor = device
             device._set_parent(self, domain="FA")
 
     @property
-    def hotwater_valve(self) -> Device:
-        return self._evo._dhw_valve
+    def sensor(self) -> Device:  # self._dhw_sensor
+        return self._dhw_sensor
+
+    def _set_dhw_valve(self, device: Device) -> None:
+        """Set the hotwater valve relay for this DHW system (13: only)."""
+
+        """Blah it now.
+
+        Check and Verb the DHW sensor (07:) of this system/CTL (if there is one).
+
+        There is only 1 way to eavesdrop a controller's DHW sensor:
+        1.  The 10A0 RQ/RP *from/to a 07:* (1x/4h)
+
+        The RQ is initiated by the DHW, so is not authorative (the CTL will RP any RQ).
+        The I/1260 is not to/from a controller, so is not useful.
+        """  # noqa: D402
+
+        # 07:38:39.124 047 RQ --- 07:030741 01:102458 --:------ 10A0 006 00181F0003E4
+        # 07:38:39.140 062 RP --- 01:102458 07:030741 --:------ 10A0 006 0018380003E8
+
+        # if "10A0" in self._msgs:
+        #     return self._msgs["10A0"].dst.addr
+        if not isinstance(device, Device) or device.type != "13":
+            raise TypeError(f"{ATTR_DHW_VALVE} can't be: {device}")
+
+        if self._dhw_valve is not None:
+            if self._dhw_valve is device:
+                return
+            raise CorruptStateError(
+                f"{ATTR_DHW_VALVE} shouldn't change: {self._dhw_valve} to {device}"
+            )
+
+        if self._dhw_valve is None:
+            self._dhw_valve = device
+            device._set_parent(self, domain="FA")
 
     @property
-    def heating_valve(self) -> Device:
-        return self._evo._htg_valve
+    def hotwater_valve(self) -> Device:  # self._dhw_valve
+        return self._dhw_valve
+
+    def _set_htg_valve(self, device: Device) -> None:  # self._htg_valve
+        """Set the heating valve relay for this DHW system (13: only)."""
+
+        if not isinstance(device, Device) or device.type != "13":
+            raise TypeError(f"{ATTR_DHW_VALVE_HTG} can't be: {device}")
+
+        if self._htg_valve is not None:
+            if self._htg_valve is device:
+                return
+            raise CorruptStateError(
+                f"{ATTR_DHW_VALVE_HTG} shouldn't change: {self._htg_valve} to {device}"
+            )
+
+        if self._htg_valve is None:
+            self._htg_valve = device
+            device._set_parent(self, domain="F9")
 
     @property
-    def relay_demand(self) -> Optional[float]:  # 0008
-        return _payload(self._relay_demand, "relay_demand")
-
-    @property  # only seen with FC, but seems should pair with 0008?
-    def relay_failsafe(self) -> Optional[float]:  # 0009
-        return _payload(self._relay_failsafe, "relay_failsafe")
-
-    @property
-    def dhw_config(self) -> Optional[dict]:  # 10A0
-        return _payload(self._dhw_params)
-
-    @property
-    def mode(self) -> Optional[dict]:  # 1F41
-        return _payload(self._dhw_mode)
+    def heating_valve(self) -> Device:  # self._htg_valve
+        return self._htg_valve
 
     @property
     def name(self) -> str:
         return "Stored HW"
 
+    @property
+    def config(self) -> Optional[dict]:  # 10A0
+        result = self._msg_payload(self._dhw_params)
+        if result:
+            return {k: v for k, v in result.items() if k != "dhw_idx"}
+
+    @property
+    def mode(self) -> Optional[dict]:  # 1F41
+        # active (setpoint), mode, until
+        return self._msg_payload(self._dhw_mode)
+
     @property
     def setpoint(self) -> Optional[float]:  # 1F41
-        return _payload(self._dhw_mode, "setpoint")
+        return self._msg_payload(self._dhw_params, ATTR_SETPOINT)
+
+    @setpoint.setter
+    def setpoint(self, value) -> None:  # 1F41
+        return self.set_config(setpoint=value)
 
     @property
     def temperature(self) -> Optional[float]:  # 1260
-        return _payload(self._temp, "temperature")
+        return self._msg_payload(self._dhw_temp, ATTR_TEMP)
 
     @property
-    def schema(self) -> dict:
-        """Return the stored HW's schema."""
+    def heat_demand(self) -> Optional[float]:  # 3150
+        return self._msg_payload(self._heat_demand, ATTR_HEAT_DEMAND)
 
-        return {
-            ATTR_DHW_SENSOR: self.sensor.id if self.sensor else None,
-            ATTR_DHW_VALVE: self.hotwater_valve.id if self.hotwater_valve else None,
-            ATTR_DHW_VALVE_HTG: self.heating_valve.id if self.heating_valve else None,
-        }
+    @property
+    def relay_demand(self) -> Optional[float]:  # 0008
+        return self._msg_payload(self._relay_demand, "relay_demand")
 
-    @property  # setpoint, config, mode (not schedule)
-    def params(self) -> dict:
-        return {
-            "dhw_params": self._dhw_params,
-        }
+    @property  # only seen with FC, but seems should pair with 0008?
+    def relay_failsafe(self) -> Optional[float]:  # 0009
+        return self._msg_payload(self._relay_failsafe, "relay_failsafe")
 
-    @property  # temp, open_windows
-    def status(self) -> dict:
-        return {
-            "temperature": self._temp,
-            "dhw_mode": self._dhw_mode,
-        }
+    def set_mode(self, mode=None, active=None, until=None) -> Task:
+        """Set the DHW mode (mode, active, until)."""
+        cmd = Command.set_dhw_mode(self._ctl.id, mode=mode, active=active, until=until)
+        return self._gwy.send_cmd(cmd)
 
-    async def cancel_override(self) -> bool:  # 1F41
-        """Reset the DHW to follow its schedule."""
-        raise NotImplementedError
+    def set_boost_mode(self) -> Task:
+        """Enable DHW for an hour, despite any schedule."""
+        return self.set_mode(
+            mode="temporary_override", active=True, until=dt.now() + td(hours=1)
+        )
 
-    async def set_override(self, mode=None, state=None, until=None) -> bool:
+    def reset_mode(self) -> Task:  # 1F41
+        """Revert the DHW to following its schedule."""
+        return self.set_mode(mode="follow_schedule")
+
+    def set_config(self, setpoint=None, overrun=None, differential=None) -> Task:
+        """Set the DHW parameters (setpoint, overrun, differential)."""
+        # if self._dhw_params:  # 10A0
+        # if setpoint is None:
+        #     setpoint = self._msg_payload(self._dhw_params, ATTR_SETPOINT)
+        # if overrun is None:
+        #     overrun = self._msg_payload(self._dhw_params, "overrun")
+        # if differential is None:
+        #     setpoint = self._msg_payload(self._dhw_params, "differential")
+
+        cmd = Command.set_dhw_params(self._ctl.id, setpoint, overrun, differential)
+        return self._gwy.send_cmd(cmd)
+
+    def reset_config(self) -> Task:  # 10A0
+        """Reset the DHW parameters to their default values."""
+        return self.set_config(setpoint=50, overrun=5, differential=1)
+
+    async def _old_set_override(self, mode=None, state=None, until=None) -> bool:
         """Force the DHW on/off for a duration, or indefinitely.
 
         Use until = ? for 1hr boost (obligates on)
         Use until = ? for until next scheduled on/off
         Use until = None for indefinitely
         """
-        # 053  I --- 01:145038 --:------ 01:145038 1F41 012 00 01 04 FFFFFF 1E061B0607E4
-        # 048  I --- 01:145038 --:------ 01:145038 1F41 012 00 00 04 FFFFFF 1E061B0607E4
+        # cmd = Command.get_dhw_mode(self._ctl.id, self.idx, mode, setpoint, until)
+        # self._gwy.send_cmd(cmd)
 
         # if mode is None and until is None:
         #     mode = "00" if setpoint is None else "02"  # Follow, Permanent
@@ -395,33 +452,44 @@ class DhwZone(ZoneBase, HeatDemand):
         self._send_cmd("1F41", verb=" W", payload=payload)
         return False
 
-    async def reset_config(self) -> bool:  # 10A0
-        """Reset the DHW parameters to their default values."""
-        raise NotImplementedError
+    @property
+    def schema(self) -> dict:
+        """Return the DHW's schema (devices)."""
+        return {
+            ATTR_DHW_SENSOR: self.sensor.id if self.sensor else None,
+            ATTR_DHW_VALVE: self.hotwater_valve.id if self.hotwater_valve else None,
+            ATTR_DHW_VALVE_HTG: self.heating_valve.id if self.heating_valve else None,
+        }
 
-    async def set_config(self, setpoint, overrun=None, differential=None) -> bool:
-        """Set the DHW parameters."""
-        raise NotImplementedError
+    @property
+    def params(self) -> dict:
+        """Return the DHW's configuration (excl. schedule)."""
+        return {a: getattr(self, a) for a in ("config", "mode")}
+
+    @property
+    def status(self) -> dict:
+        """Return the DHW's current state."""
+        return {a: getattr(self, a) for a in (ATTR_TEMP, ATTR_HEAT_DEMAND)}
 
 
-class ZoneSchedule:
+class ZoneSchedule:  # 0404
     """Evohome zones have a schedule."""
 
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
+
         self._schedule = Schedule(self)
 
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
 
-        if discover_flag & DISCOVER_STATUS:
-            # asyncio.create_task(self.get_schedule())  # 0404
-            pass
+        if False and discover_flag & DISCOVER_STATUS:  # TODO: add back in
+            self._loop.create_task(self.get_schedule())  # 0404
 
     def _handle_msg(self, msg) -> bool:
         super()._handle_msg(msg)
 
         if msg.code == "0404" and msg.verb == "RP":
-            _LOGGER.debug("Zone(%s).update: Received RP/0404 (schedule)", self.id)
+            _LOGGER.debug("Zone(%s): Received RP/0404 (schedule) pkt", self)
 
     async def get_schedule(self, force_refresh=None) -> Optional[dict]:
         schedule = await self._schedule.get_schedule(force_refresh=force_refresh)
@@ -464,16 +532,18 @@ class Zone(ZoneSchedule, ZoneBase):
         self.device_by_id = {}
         self._sensor = None
 
+        self._heat_demand = None  # Not used by ELE
         self._mode = None
         # self._name = None  # from super()
         self._setpoint = None
-        self._temp = None
+        self._temperature = None
         self._zone_config = None
 
         # these needed here, as we can't use __init__
-        self._mix_config = None
-        self._ufh_setpoint = None
-        self._window_open = None
+        self._mix_config = None  # MIX
+        self._ufh_setpoint = None  # UFH
+        self._window_open = None  # RAD (or: ALL)
+        self._actuator_state = None  # ELE, ZON
 
         self._schedule = Schedule(self)
 
@@ -483,8 +553,9 @@ class Zone(ZoneSchedule, ZoneBase):
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
-        if DEV_MODE and self.idx == "99":  # dev/test code
-            asyncio.create_task(  # TODO: test/dev only
+        # HACK: dev/test code
+        if DEV_MODE and self.idx == "99":
+            self._loop.create_task(
                 self.async_cancel_override()
                 # self.async_set_override(
                 #     setpoint=15.9,
@@ -494,18 +565,24 @@ class Zone(ZoneSchedule, ZoneBase):
             )
 
         # TODO: add code to determine zone type if it doesn't have one, using 0005s
+        if discover_flag & DISCOVER_SCHEMA:
+            [  # 000C: find the sensor and the actuators, if any
+                self._send_cmd("000C", payload=f"{self.idx}{dev_type}")
+                for dev_type in ("00", "04")  # CODE_000C_ZONE_TYPE
+            ]
 
-        [  # 000C: find the sensor and the actuators, if any
-            self._send_cmd("000C", payload=f"{self.idx}{dev_type}")
-            for dev_type in ("00", "04")  # CODE_000C_ZONE_TYPE
-        ]  # TODO: use 08, not 00
+        if discover_flag & DISCOVER_PARAMS:
+            self._gwy.send_cmd(Command.get_zone_config(self._ctl.id, self.idx))  # 000A
+            self._gwy.send_cmd(Command.get_zone_name(self._ctl.id, self.idx))  # 0004
+
+        if discover_flag & DISCOVER_STATUS:
+            self._gwy.send_cmd(Command.get_zone_mode(self._ctl.id, self.idx))  # 2349
+            for code in ("12B0", "30C9"):  # sadly, no 3150
+                self._send_cmd(code)  # , payload=self.idx)
 
         # start collecting the schedule
         # self._schedule.req_schedule()  # , restart=True) start collecting schedule
 
-        for code in ("0004", "000A", "2349", "30C9"):  # sadly, no 3150
-            self._send_cmd(code)  # , payload=self.idx)
-
     def _handle_msg(self, msg) -> bool:
         super()._handle_msg(msg)
 
@@ -518,7 +595,7 @@ class Zone(ZoneSchedule, ZoneBase):
         # not UFH (it seems), but ELE or VAL; and possibly a MIX support 0008 too
         elif msg.code in ("0008", "0009"):  # TODO: how to determine is/isn't MIX?
             assert msg.src.type in ("01", "13"), msg.src.type  # 01 as a stat
-            assert self._zone_type in (None, "ELE", "VAL"), self._zone_type
+            assert self._zone_type in (None, "ELE", "VAL", "MIX"), self._zone_type
 
             if self._zone_type is None:
                 self._set_zone_type("ELE")  # might eventually be: "VAL"
@@ -526,18 +603,21 @@ class Zone(ZoneSchedule, ZoneBase):
         elif msg.code == "000A":
             self._zone_config = msg
 
+        elif msg.code == "12B0":
+            self._window_open = msg
+
         elif msg.code == "2309" and msg.verb in (" I", "RP"):  # setpoint
-            assert msg.src.type == "01", "coding error"
+            assert msg.src.type == "01", "coding error zxw"
             self._setpoint = msg
 
         elif msg.code == "2349" and msg.verb in (" I", "RP"):  # mode, setpoint
-            assert msg.src.type == "01", "coding error"
+            assert msg.src.type == "01", "coding error zxx"
             self._mode = msg
             self._setpoint = msg
 
         elif msg.code == "30C9" and msg.verb in (" I", "RP"):  # used by sensor matching
             assert msg.src.type in DEVICE_HAS_ZONE_SENSOR + ("01",), "coding error"
-            self._temp = msg
+            self._temperature = msg
 
         elif msg.code == "3150":  # TODO: and msg.verb in (" I", "RP")?
             assert msg.src.type in ("00", "02", "04", "13")
@@ -633,48 +713,51 @@ class Zone(ZoneSchedule, ZoneBase):
         self._discover()  # TODO: needs tidyup (ref #67)
 
     @property
-    def heat_demand(self) -> Optional[float]:
-        demands = [
-            d.heat_demand
-            for d in self.devices
-            if hasattr(d, "heat_demand") and d.heat_demand is not None
-        ]
-        # return max(demands) if demands else None
-        return round(sum(demands) / len(demands), 1) if demands else None
+    def name(self) -> Optional[str]:
+        """Return the name of the zone."""
+        return self._msg_payload(self._name, "name")
+
+    @name.setter
+    def name(self, value) -> Optional[str]:
+        """Set the name of the zone."""
+        self._gwy.send_cmd(Command.set_zone_name(self._ctl.id, self.idx, value))
+
+        # async def get_name(self, force_refresh=None) -> Optional[str]:
+        #     """Return the name of the zone."""
+        #     if not force_refresh and self._name is not None:
+        #         return self._msg_payload(self._name, "name")
+
+        #     self._name = None
+        #     self._send_cmd("0004", payload=f"{self.idx}00")
+        #     while self._name is None:
+        #         await asyncio.sleep(0.05)
+
+        #     return self._msg_payload(self._name, "name")
+
+    @property
+    def config(self) -> Optional[dict]:  # 000A
+        if not self._zone_config or self._zone_config.is_expired:
+            return
+
+        elif isinstance(self._zone_config.payload, dict):
+            result = self._msg_payload(self._zone_config)
+
+        elif isinstance(self._zone_config.payload, list):
+            tmp = [z for z in self._zone_config.payload if z["zone_idx"] == self.idx]
+            result = {k: v for k, v in tmp[0].items() if k[:1] != "_"}
+
+        if result:
+            return {k: v for k, v in result.items() if k != "zone_idx"}
 
     @property
     def mode(self) -> Optional[dict]:  # 2349
+        # setpoint, mode, until
         if not self._mode or self._mode.is_expired:
             return
 
-        elif isinstance(self._mode.payload, dict):
-            return _payload(self._mode)
-
-        elif isinstance(self._mode.payload, list):
-            tmp = [z for z in self._mode.payload if z["zone_idx"] == self.idx]
-            return {k: v for k, v in tmp[0].items() if k[:1] != "_" and k != "zone_idx"}
-
-    # async def get_name(self, force_refresh=None) -> Optional[str]:
-    #     """Return the name of the zone."""
-    #     if not force_refresh and self._name is not None:
-    #         return _payload(self._name, "name")
-
-    #     self._name = None
-    #     self._send_cmd("0004", payload=f"{self.idx}00")
-    #     while self._name is None:
-    #         await asyncio.sleep(0.05)
-
-    #     return _payload(self._name, "name")
-
-    @property
-    def name(self) -> Optional[str]:
-        """Return the name of the zone."""
-        return _payload(self._name, "name")
-
-    # @name.setter
-    # def name(self, value) -> Optional[str]:
-    #     """Set the name of the zone."""
-    #     return
+        result = self._msg_payload(self._mode)
+        if result:
+            return {k: v for k, v in result.items() if k != "zone_idx"}
 
     @property
     def setpoint(self) -> Optional[float]:  # 2309 (2349 is a superset of 2309)
@@ -682,129 +765,107 @@ class Zone(ZoneSchedule, ZoneBase):
             return
 
         elif isinstance(self._setpoint.payload, dict):
-            return _payload(self._setpoint, "setpoint")
+            return self._msg_payload(self._setpoint, ATTR_SETPOINT)
 
         elif isinstance(self._setpoint.payload, list):
             _zone = [z for z in self._setpoint.payload if z["zone_idx"] == self.idx]
-            return _zone[0]["setpoint"]
+            return _zone[0][ATTR_SETPOINT]
 
     @setpoint.setter
     def setpoint(self, value) -> None:
         """Set the target temperature, until the next scheduled setpoint."""
         if value is None:
-            self.cancel_override()
+            self.reset_mode()
         else:
-            self.set_override(mode="advanced_override", setpoint=value)
+            cmd = Command.set_zone_setpoint(self._ctl.id, self.idx, value)
+            self._gwy.send_cmd(cmd)
+            # NOTE: the following doesn't wotk for e.g. Hometronics
+            # self.set_mode(mode="advanced_override", setpoint=value)
 
     @property
     def temperature(self) -> Optional[float]:  # 30C9
-        if not self._temp or self._temp.is_expired:
-            if self.sensor and self.sensor.temperature:
-                self._temp = self.sensor._temp
+        # if not self._temperature or self._temperature.is_expired:
+        #     if self.sensor and self.sensor.temperature:
+        #         self._temperature = self.sensor._temp
 
-        if not self._temp or self._temp.is_expired:
+        if not self._temperature or self._temperature.is_expired:
             return
 
-        elif isinstance(self._temp.payload, dict):
-            return _payload(self._temp, "temperature")
+        elif isinstance(self._temperature.payload, dict):
+            return self._msg_payload(self._temperature, ATTR_TEMP)
 
-        elif isinstance(self._temp.payload, list):
-            tmp = [z for z in self._temp.payload if z["zone_idx"] == self.idx]
-            return tmp[0]["temperature"]
+        elif isinstance(self._temperature.payload, list):
+            tmp = [z for z in self._temperature.payload if z["zone_idx"] == self.idx]
+            return tmp[0][ATTR_TEMP]
 
     @property
-    def zone_config(self) -> Optional[dict]:  # 000A
-        if not self._zone_config or self._zone_config.is_expired:
-            return
+    def heat_demand(self) -> Optional[float]:
+        """Return an estimate of the zone's current heat demand."""
+        demands = [
+            d.heat_demand
+            for d in self.devices
+            if hasattr(d, ATTR_HEAT_DEMAND) and d.heat_demand is not None
+        ]
+        # return max(demands) if demands else None
+        return round(sum(demands) / len(demands), 1) if demands else None
 
-        elif isinstance(self._zone_config.payload, dict):
-            return _payload(self._zone_config)
+    @property
+    def window_open(self) -> Optional[bool]:  # 12B0  # TODO: don't work >1 TRV?
+        """Return an estimate of the zone's current window_open state."""
+        windows = [
+            d.window_open
+            for d in self.devices
+            if hasattr(d, ATTR_WINDOW_OPEN) and d.window_open is not None
+        ]
+        return any(windows) if windows else None
 
-        elif isinstance(self._zone_config.payload, list):
-            tmp = [z for z in self._zone_config.payload if z["zone_idx"] == self.idx]
-            return {k: v for k, v in tmp[0].items() if k[:1] != "_" and k != "zone_idx"}
+    def set_mode(self, mode=None, setpoint=None, until=None) -> Task:
+        """Override the zone's setpoint for a specified duration, or indefinitely."""
+        if all(v is None for v in (mode, setpoint, until)):
+            return self.reset_mode()
+        cmd = Command.set_zone_mode(self._ctl.id, self.idx, mode, setpoint, until)
+        return self._gwy.send_cmd(cmd)
 
-    def cancel_override(self) -> None:  # 2349
-        """Revert to following the schedule."""
-        self.set_override()
-
-    def frost_protect(self) -> None:  # 2349
+    def set_frost_mode(self) -> Task:  # 2349
         """Set the zone to the lowest possible setpoint, indefinitely."""
-        self.set_override(mode="permanent_override", setpoint=5)  # TODO
+        return self.set_mode(mode="permanent_override", setpoint=5)  # TODO
 
-    def set_override(self, mode=None, setpoint=None, until=None) -> None:
-        """Override the setpoint for a specified duration, or indefinitely.
+    def reset_mode(self) -> Task:  # 2349
+        """Revert the zone to following its schedule."""
+        return self.set_mode(mode="follow_schedule")
 
-        The setpoint has a resolution of 0.1 C. If a setpoint temperature is required,
-        but none is provided, the controller will use the maximum possible value.
+    def set_config(self, setpoint=None, overrun=None, differential=None) -> Task:
+        """Set the zone's parameters (setpoint, overrun, differential)."""
+        cmd = Command.set_zone_config(self._ctl.id, setpoint, overrun, differential)
+        return self._gwy.send_cmd(cmd)
 
-        The until has a resolution of 1 min.
+    def reset_config(self) -> Task:
+        """Reset the zone's parameters to their default values."""
+        return self.set_config(setpoint=50, overrun=5, differential=1)
 
-        Incompatible combinations:
-          - mode == Follow & setpoint not None (will silently ignore setpoint)
-          - mode == Temporary & until is None (will silently ignore)
-        """
+    def set_name(self, name) -> Task:
+        """Set the zone's name."""
+        cmd = Command.set_zone_name(self._ctl.id, self.idx, name)
+        return self._gwy.send_cmd(cmd)
 
-        if mode is not None:
-            if isinstance(mode, int):
-                mode = f"{mode:02X}"
-            elif not isinstance(mode, str):
-                raise TypeError(f"Invalid zone mode: {mode}")
-            if mode in ZONE_MODE_MAP:
-                mode = ZONE_MODE_MAP["mode"]
-            elif mode not in ZONE_MODE_LOOKUP:
-                raise TypeError(f"Unknown zone mode: {mode}")
-        elif until is None:  # mode is None
-            mode = "advanced_override" if setpoint else "follow_schedule"
-        else:  # if until is not None:
-            mode = "temporary_override" if setpoint else "advanced_override"
-
-        setpoint = _temp(setpoint)  # None means max, if a temp is required
-
-        if until is None:
-            mode = "advanced_override" if mode == "temporary_override" else mode
-
-        mode = ZONE_MODE_LOOKUP[mode]
-
-        if until is None:
-            payload = f"{self.idx}{setpoint}{mode}FFFFFF"
-        else:  # required only by temporary_override, ignored by others
-            payload = f"{self.idx}{setpoint}{mode}FFFFFF{dtm_to_hex(until)}"
-
-        self._send_cmd("2349", verb=" W", payload=payload)
-
-    @property  # id, type
+    @property
     def schema(self) -> dict:
-        """Return the zone's schema."""
-
+        """Return the zone's schema (type, devices)."""
         return {
             ATTR_ZONE_TYPE: self.heating_type,
             ATTR_ZONE_SENSOR: self._sensor.id if self._sensor else None,
             ATTR_DEVICES: [d.id for d in self.devices],
         }
 
-    @property  # setpoint, config, mode (not schedule)
+    @property  # TODO: setpoint
     def params(self) -> dict:
         """Return the zone's configuration (excl. schedule)."""
-
-        ATTR_NAME = "name"  # TODO
-        ATTR_MODE = "mode"
-        ATTR_CONFIG = "zone_config"
-
-        return {
-            ATTR_NAME: self.name,
-            ATTR_MODE: self.mode,
-            ATTR_CONFIG: self.zone_config,
-        }
+        return {a: getattr(self, a) for a in ("config", "mode", "name")}
 
     @property
     def status(self) -> dict:
         """Return the zone's current state."""
-
-        return {
-            ATTR_SETPOINT: self.setpoint,
-            ATTR_TEMP: self.temperature,
-        }
+        return {a: getattr(self, a) for a in (ATTR_SETPOINT, ATTR_TEMP)}
 
 
 class ZoneDemand:  # not all zone types call for heat
@@ -813,15 +874,15 @@ class ZoneDemand:  # not all zone types call for heat
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         super()._discover(discover_flag=discover_flag)
 
-        if discover_flag & DISCOVER_STATUS:
-            self._send_cmd("12B0")  # , payload=self.idx
+        if discover_flag & DISCOVER_STATUS:  # controller will not respond to this
+            self._send_cmd("3150")  # , payload=self.idx
 
     @property
     def heat_demand(self) -> Optional[float]:  # 3150
         demands = [
             d.heat_demand
             for d in self.devices
-            if hasattr(d, "heat_demand") and d.heat_demand is not None
+            if hasattr(d, ATTR_HEAT_DEMAND) and d.heat_demand is not None
         ]
         return max(demands + [0]) if demands else None
 
@@ -845,22 +906,25 @@ class EleZone(Zone):  # Electric zones (do *not* call for heat)
     def _handle_msg(self, msg) -> bool:
         super()._handle_msg(msg)
 
-        if msg.code == "3150":  # ZV zones are Elec zones that also call for heat
+        if msg.code == "3150":  # ZON zones are ELE zones that also call for heat
+            self._heat_demand = msg  # 3150
             self._set_zone_type("VAL")
 
+        elif msg.code == "3EF0":
+            self._actuator_state = msg  # 3EF0
+
     @property
-    def actuator_enabled(self) -> Optional[bool]:  # 3EF0
-        return self._get_msg_value("3EF0")  # , "actuator_enabled"
+    def enabled(self) -> Optional[bool]:  # 3EF0
+        return self._msg_payload(self._actuator_state, "actuator_enabled")
 
     @property
     def actuator_state(self) -> Optional[float]:  # 3EF1
-        return self._get_msg_value("3EF1")
+        return self._msg_payload(self._actuator_state)
 
     @property
     def status(self) -> dict:
         return {
             **super().status,
-            "actuator_enabled": self.actuator_enabled,
             "actuator_state": self.actuator_state,
         }
 
@@ -884,23 +948,6 @@ class RadZone(ZoneDemand, Zone):
         # super()._discover(discover_flag=discover_flag)
         self._send_cmd("000C", payload=f"{self.idx}08")
 
-    def _handle_msg(self, msg) -> bool:
-        super()._handle_msg(msg)
-
-        if msg.code == "12B0":
-            self._window_open = msg
-
-    @property
-    def window_open(self) -> Optional[bool]:  # 12B0
-        return _payload(self._window_open, "window_open")  # TODO: doesn't work >1 TRV
-
-    @property
-    def status(self) -> dict:
-        return {
-            **super().status,
-            ATTR_OPEN_WINDOW: self.window_open,
-        }
-
 
 class UfhZone(ZoneDemand, Zone):
     """For underfloor heating controlled by an HCE80/HCC80 (will also call for heat)."""
@@ -919,7 +966,7 @@ class UfhZone(ZoneDemand, Zone):
 
     @property
     def ufh_setpoint(self) -> Optional[float]:  # 22C9
-        return _payload(self._ufh_setpoint, "setpoint")
+        return self._msg_payload(self._ufh_setpoint, ATTR_SETPOINT)
 
     @property
     def status(self) -> dict:
@@ -946,18 +993,21 @@ class MixZone(Zone):
 
     @property
     def mix_config(self) -> dict:
-        return _payload(self._mix_config)
+        return self._msg_payload(self._mix_config)
 
     @property
     def params(self) -> dict:
-        return {**super().status, "mix_config": self.mix_config}
+        return {
+            **super().status,
+            "mix_config": self.mix_config,
+        }
 
 
 ZONE_CLASSES = {
-    "RAD": RadZone,
-    "ELE": EleZone,
-    "VAL": ValZone,
-    "UFH": UfhZone,
-    "MIX": MixZone,
-    "DHW": DhwZone,
+    Zone.RAD: RadZone,
+    Zone.ELE: EleZone,
+    Zone.VAL: ValZone,
+    Zone.UFH: UfhZone,
+    Zone.MIX: MixZone,
+    Zone.DHW: DhwZone,
 }
